%%% Demonstrate Robust M-Estimation on 1D synthetic data.
%%% Compares Least Squares (LS), Least Absolute Error (L1),
%%% Geman-McLure (GM), and Lorentzian (LZ) estimators. 

%% ADJ, March 2005.
clear;
%close all;

seed = round(sum(1000*clock));
rand('state',seed);
randn('state',seed);

%%% Test Data Parameters
nSamp = 100;     % total number of data observations
meanIn = 100;    % mean of inliers and outliers
meanOut = 3*meanIn; % mean of outliers
sigmaIn = 10;    % stdev of inliers
sigmaOut = 100;  % stdev of outliers
fracOut = 0.6;   % fraction of data which are outliers  TRY: 0.05 0.1 0.3 0.6
sigmaRho = sigmaIn;  % robust estimator sigma

%%%%%%  Generate Data
out = rand(nSamp,1) < fracOut;
data = meanIn + sigmaIn * randn(nSamp,1);
% overwrite outliers with new samples
data(out) = meanOut + sigmaOut * randn(sum(out),1);

sampleMean = sum(data)/nSamp;
sampleMedian = median(data);
[sampleMean sampleMedian]

%%%%%% Plot data and sample mean/median.
[n, x] = histo(data, 20);
figure(1); clf; bar(x,n);
hold on;
ax = axis;
plot([sampleMean sampleMean], [ax(3), ax(4)], 'r', 'LineWidth', 2.0);
plot([sampleMedian sampleMedian], [ax(3), ax(4)], 'g', 'LineWidth', 2.0);
title('Data histogram (sample mean (r), median (g))');
xlabel('Data Value (x)');
ylabel('Frequency');

%%%%%% Compute objective functions for different estimators
xRange = ax(1):0.1:ax(2);

err = repmat(data, 1, length(xRange)) - repmat(xRange, length(data), 1);
rhoErr = err.^2 ./ (sigmaRho.^2 + err.^2);
objFunGM = sum(rhoErr, 1);
objFunLS = sum(err.^2,1);
rhoErr = log(1 + 0.5/sigmaRho^2 * err.^2);
objFunLZ = sum(rhoErr,1);
objFunL1 = sum(abs(err),1);

%%%%%% Plot objective functions for different estimators

figure(2); clf;
subplot(2,2,1);
plot(xRange, objFunGM, 'b');
% Report discrete minimum
[mn, kmn] = min(objFunGM);
title(sprintf('GM (%5.1f)',xRange(kmn)))
xlabel('a');
ylabel('Objective Function');
subplot(2,2,2);
plot(xRange, objFunLS, 'b');
% Report discrete minimum
[mn, kmn] = min(objFunLS);
title(sprintf('LS (%5.1f)',xRange(kmn)))
xlabel('a');
subplot(2,2,3);
plot(xRange, objFunLZ, 'b');
% Report discrete minimum
[mn, kmn] = min(objFunLZ);
[xRange(kmn) mn];
title(sprintf('LZ (%5.1f)',xRange(kmn)))
xlabel('a');
ylabel('Objective Function');
subplot(2,2,4);
plot(xRange, objFunL1, 'b');
% Report discrete minimum
[mn, kmn] = min(objFunL1);
[xRange(kmn) mn];
title(sprintf('L1 (%5.1f)',xRange(kmn)))
xlabel('a');

%%%%%% Try out simple descent
bestVals = [inf inf inf inf];
bestArgs = [nan nan nan nan];
for trials = 1:10
  %% Use random sample from data for initial guess
  a0 = data(1+floor(rand(1)*length(data)));
  [valLS, minArgLS, okLS, nLS] = simple1DDescent(data, a0, sigmaRho, 'LS');

  [valGM, minArgGM, okGM, nGM] = simple1DDescent(data, a0, sigmaRho, 'GM');

  [valLZ, minArgLZ, okLZ, nLZ] = simple1DDescent(data, a0, sigmaRho, 'LZ');

  [valL1, minArgL1, okL1, nL1] = simple1DDescent(data, a0, sigmaRho, 'L1');

  %% Dump results...
  a0
  [nGM nLS nLZ nL1]
  [minArgGM minArgLS minArgLZ minArgL1]
  
  %% Keep best results
  newVals = [valGM valLS valLZ valL1];
  newArgs = [minArgGM minArgLS minArgLZ minArgL1];
  bestVals = min(bestVals, newVals);
  idx = (bestVals == newVals);
  if any(idx(:))
    bestArgs(idx) = newArgs(idx);
  end 
end

%% Report the best arguments obtained for GM, LS, LZ (respectively)
bestArgs
%% These results should match with the discrete minima reported in Fig.2

%% For comparison
[sampleMean sampleMedian]