function [oa, a, cnvrg, it] = simple1DDescent(data, a, sigma, objectiveName);
%  [mnVal, minArg, cnvrged, nIts] = simple1DDescent(a0, sigma, objectiveName);
% Input:
%    data - vector of data samples
%    a0   - initial guess for robust mean.
%    sigma   - scale parameter for robust estimator 
%    objectiveName =  'GM' Geman-McLure, 'LZ' Lorentzian,  
%                     'L1' abs error (or 'LS' squared error)
%  Output:
%    oa  - objective function at current state "a"
%    a   - current state
%    cnvrg - method converged, so current state is (roughly) a 
%            local minimum of the objective function.
%    it  - number of iterations used.
  
  data = data(:);
  tol = 1.0e-2;
  step = sigma;
  maxIts = 1000;
  
  err = data - a;
  if strcmpi('GM',objectiveName)
    oa = sum(err.^2 ./(sigma^2 + err.^2));
  elseif strcmpi('LZ',objectiveName)
    oa = sum(log(1+(0.5/sigma.^2)* err.^2));
  elseif strcmpi('L1',objectiveName)
    oa = sum(abs(err));
  else 
    oa = sum(err.^2);
  end

  for it=0:maxIts
    err = data - a;
    
    if strcmpi('GM',objectiveName)
      g = -sum(2*sigma^2 * err ./(sigma^2 + err.^2).^2);
    elseif strcmpi('LZ',objectiveName)
      g = -sum(2* err ./(2*sigma^2 + err.^2));
    elseif strcmpi('L1',objectiveName)
      g = -(length(data)-2*sum(double(err<0)));
    else 
      g = -sum(2 * err);
    end
    
    while step > tol
      t = a - step * sign(g);

      err = data - t;
      if strcmpi('GM',objectiveName)
        ot = sum(err.^2 ./(sigma^2 + err.^2));
      elseif strcmpi('LZ',objectiveName)
        ot = sum(log(1+(0.5/sigma.^2)* err.^2));
      elseif strcmpi('L1',objectiveName)
        ot = sum(abs(err));
      else 
        ot = sum(err.^2);
      end

      if ot < oa
        %% Accept step
        oa = ot;
        a = t;
        break;
      else
        step = step/2;
      end
    end
    if (step <= tol)
      break;
    end

  end
   
  cnvrg = (step <= tol);
  return;
  